@@ -111,6 +111,49 @@ Rcpp::List regression_predict_oob(Rcpp::List forest_object,
   return result;
 }
 
+// [[Rcpp::export]]
+Rcpp::List ll_regression_train(Rcpp::NumericMatrix train_matrix,
+                            Eigen::SparseMatrix<double> sparse_train_matrix,
+                            size_t outcome_index,
+                            size_t sample_weight_index,
+                            double split_lambda,
+                            bool weight_penalty,
+                            std::vector<size_t> ll_split_variables,
+                            size_t ll_split_cutoff,
+                            std::vector<double> overall_beta,
+                            bool use_sample_weights,
+                            unsigned int mtry,
+                            unsigned int num_trees,
+                            unsigned int min_node_size,
+                            double sample_fraction,
+                            bool honesty,
+                            double honesty_fraction,
+                            bool honesty_prune_leaves,
+                            size_t ci_group_size,
+                            double alpha,
+                            double imbalance_penalty,
+                            std::vector<size_t> clusters,
+                            unsigned int samples_per_cluster,
+                            unsigned int num_threads,
+                            unsigned int seed) {
+  ForestTrainer trainer = ll_regression_trainer(split_lambda, weight_penalty, overall_beta,
+                                               ll_split_cutoff, ll_split_variables);
+
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  data->set_outcome_index(outcome_index - 1);
+  if(use_sample_weights) {
+    data->set_weight_index(sample_weight_index - 1);
+  }
+  data->sort();
+
+  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size, honesty,
+                        honesty_fraction, honesty_prune_leaves, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
+  Forest forest = trainer.train(*data, options);
+
+  std::vector<Prediction> predictions;
+  return RcppUtilities::create_forest_object(forest, predictions);
+}
+
 // [[Rcpp::export]]
 Rcpp::List ll_regression_predict(Rcpp::List forest,
                                 Rcpp::NumericMatrix train_matrix,
