@@ -24,21 +24,20 @@
 #include "DefaultData.h"
 #include "utility.h"
 
-DefaultData::DefaultData() :
-    DefaultData(NULL, 0, 0) {}
+namespace grf {
 
-DefaultData::DefaultData(double* data,
-                         size_t num_rows,
-                         size_t num_cols) :
-    data(data) {
-  this->num_rows = num_rows;
-  this->num_cols = num_cols;
+DefaultData::DefaultData() {
+  this->data = std::vector<double>();
+  this->num_rows = 0;
+  this->num_cols = 0;
 }
 
-DefaultData::~DefaultData() {
-  if (!external_data) {
-    delete[] data;
-  }
+DefaultData::DefaultData(const std::vector<double>& data,
+                         size_t num_rows,
+                         size_t num_cols) {
+  this->data = std::move(data);
+  this->num_rows = num_rows;
+  this->num_cols = num_cols;
 }
 
 double DefaultData::get(size_t row, size_t col) const {
@@ -46,9 +45,11 @@ double DefaultData::get(size_t row, size_t col) const {
 }
 
 void DefaultData::reserve_memory() {
-  data = new double[num_cols * num_rows];
+  data.resize(num_cols * num_rows);
 }
 
 void DefaultData::set(size_t col, size_t row, double value, bool& error) {
   data[col * num_rows + row] = value;
 }
+
+} // namespace grf
