@@ -23,24 +23,25 @@
 #include "commons/DefaultData.h"
 #include "prediction/QuantilePredictionStrategy.h"
 
+namespace grf {
+
 QuantilePredictionStrategy::QuantilePredictionStrategy(std::vector<double> quantiles):
     quantiles(quantiles) {
 };
 
-size_t QuantilePredictionStrategy::prediction_length() {
+size_t QuantilePredictionStrategy::prediction_length() const {
     return quantiles.size();
 }
 
 std::vector<double> QuantilePredictionStrategy::predict(
     size_t prediction_sample,
     const std::unordered_map<size_t, double>& weights_by_sample,
-    const Data* train_data,
-    const Data* data) {
+    const Data& train_data,
+    const Data& data) const {
   std::vector<std::pair<size_t, double>> samples_and_values;
-  for (auto it = weights_by_sample.begin(); it != weights_by_sample.end(); it++) {
-    size_t sample = it->first;
-    samples_and_values.push_back(std::pair<size_t, double>(
-        sample, train_data->get_outcome(sample)));
+  for (const auto& entry : weights_by_sample) {
+    size_t sample = entry.first;
+    samples_and_values.emplace_back(sample, train_data.get_outcome(sample));
   }
 
   return compute_quantile_cutoffs(weights_by_sample, samples_and_values);
@@ -48,20 +49,24 @@ std::vector<double> QuantilePredictionStrategy::predict(
 
 std::vector<double> QuantilePredictionStrategy::compute_quantile_cutoffs(
     const std::unordered_map<size_t, double>& weights_by_sample,
-    std::vector<std::pair<size_t, double>>& samples_and_values) {
+    std::vector<std::pair<size_t, double>>& samples_and_values) const {
   std::sort(samples_and_values.begin(),
             samples_and_values.end(),
             [](std::pair<size_t, double> first_pair, std::pair<size_t, double> second_pair) {
-              return first_pair.second < second_pair.second;
+              // Note: we add a tie-breaker here to ensure that this sort consistently produces the
+              // same element ordering. Otherwise, different runs of the algorithm could result in
+              // different quantile predictions on the same data.
+              return first_pair.second < second_pair.second
+                  || (first_pair.second == second_pair.second && first_pair.first < second_pair.first);
             });
 
   std::vector<double> quantile_cutoffs;
   auto quantile_it = quantiles.begin();
   double cumulative_weight = 0.0;
 
-  for (auto it = samples_and_values.begin(); it != samples_and_values.end(); ++it) {
-    size_t sample = it->first;
-    double value = it->second;
+  for (const auto& entry : samples_and_values) {
+    size_t sample = entry.first;
+    double value = entry.second;
 
     cumulative_weight += weights_by_sample.at(sample);
     while (quantile_it != quantiles.end() && cumulative_weight >= *quantile_it) {
@@ -79,10 +84,12 @@ std::vector<double> QuantilePredictionStrategy::compute_quantile_cutoffs(
 
 std::vector<double> QuantilePredictionStrategy::compute_variance(
     size_t sampleID,
-    std::vector<std::vector<size_t>> samples_by_tree,
-    std::unordered_map<size_t, double> weights_by_sampleID,
-    const Data* train_data,
-    const Data* data,
-    size_t ci_group_size){
+    const std::vector<std::vector<size_t>>& samples_by_tree,
+    const std::unordered_map<size_t, double>& weights_by_sampleID,
+    const Data& train_data,
+    const Data& data,
+    size_t ci_group_size) const {
   return { 0.0 };
 }
+
+} // namespace grf
