@@ -16,6 +16,7 @@
  #-------------------------------------------------------------------------------*/
 
 #include <algorithm>
+#include <cmath>
 
 #include "InstrumentalSplittingRule.h"
 
@@ -29,6 +30,7 @@ InstrumentalSplittingRule::InstrumentalSplittingRule(size_t max_num_unique_value
     alpha(alpha),
     imbalance_penalty(imbalance_penalty) {
   this->counter = new size_t[max_num_unique_values];
+  this->weight_sums = new double[max_num_unique_values];
   this->sums = new double[max_num_unique_values];
   this->num_small_z = new size_t[max_num_unique_values];
   this->sums_z = new double[max_num_unique_values];
@@ -39,6 +41,9 @@ InstrumentalSplittingRule::~InstrumentalSplittingRule() {
   if (counter != nullptr) {
     delete[] counter;
   }
+  if (weight_sums != nullptr) {
+    delete[] weight_sums;
+  }
   if (sums != nullptr) {
     delete[] sums;
   }
@@ -59,25 +64,29 @@ bool InstrumentalSplittingRule::find_best_split(const Data& data,
                                                 const std::vector<double>& responses_by_sample,
                                                 const std::vector<std::vector<size_t>>& samples,
                                                 std::vector<size_t>& split_vars,
-                                                std::vector<double>& split_values) {
+                                                std::vector<double>& split_values,
+                                                std::vector<bool>& send_missing_left) {
   size_t num_samples = samples[node].size();
 
   // Precompute relevant quantities for this node.
+  double weight_sum_node = 0.0;
   double sum_node = 0.0;
   double sum_node_z = 0.0;
   double sum_node_z_squared = 0.0;
   for (auto& sample : samples[node]) {
-    sum_node += responses_by_sample[sample];
+    double sample_weight = data.get_weight(sample);
+    weight_sum_node += sample_weight;
+    sum_node += sample_weight * responses_by_sample[sample];
 
     double z = data.get_instrument(sample);
-    sum_node_z += z;
-    sum_node_z_squared += z * z;
+    sum_node_z += sample_weight * z;
+    sum_node_z_squared += sample_weight * z * z;
   }
 
-  double size_node = sum_node_z_squared - sum_node_z * sum_node_z / (double) num_samples;
+  double size_node = sum_node_z_squared - sum_node_z * sum_node_z / weight_sum_node;
   double min_child_size = size_node * alpha;
 
-  double mean_z_node = sum_node_z / num_samples;
+  double mean_z_node = sum_node_z / weight_sum_node;
   size_t num_node_small_z = 0;
   for (auto& sample : samples[node]) {
     double z = data.get_instrument(sample);
@@ -90,16 +99,17 @@ bool InstrumentalSplittingRule::find_best_split(const Data& data,
   size_t best_var = 0;
   double best_value = 0;
   double best_decrease = 0.0;
+  bool best_send_missing_left = true;
 
   for (auto& var : possible_split_vars) {
     // Use faster method for both cases
     double q = (double) num_samples / (double) data.get_num_unique_data_values(var);
-    if (q < Q_THRESHOLD) {
-      find_best_split_value_small_q(data, node, var, num_samples, sum_node, mean_z_node, num_node_small_z,
+    if (q < Q_THRESHOLD || data.contains_nan()) {
+      find_best_split_value_small_q(data, node, var, num_samples, weight_sum_node, sum_node, mean_z_node, num_node_small_z,
                                     sum_node_z, sum_node_z_squared, min_child_size, best_value,
-                                    best_var, best_decrease, responses_by_sample, samples);
+                                    best_var, best_decrease, best_send_missing_left, responses_by_sample, samples);
     } else {
-      find_best_split_value_large_q(data, node, var, num_samples, sum_node, mean_z_node, num_node_small_z,
+      find_best_split_value_large_q(data, node, var, num_samples, weight_sum_node, sum_node, mean_z_node, num_node_small_z,
                                     sum_node_z, sum_node_z_squared, min_child_size, best_value,
                                     best_var, best_decrease, responses_by_sample, samples);
     }
@@ -113,12 +123,14 @@ bool InstrumentalSplittingRule::find_best_split(const Data& data,
   // Save best values
   split_vars[node] = best_var;
   split_values[node] = best_value;
+  send_missing_left[node] = best_send_missing_left;
   return false;
 }
 
 void InstrumentalSplittingRule::find_best_split_value_small_q(const Data& data,
                                                               size_t node, size_t var,
                                                               size_t num_samples,
+                                                              double weight_sum_node,
                                                               double sum_node,
                                                               double mean_node_z,
                                                               size_t num_node_small_z,
@@ -128,103 +140,156 @@ void InstrumentalSplittingRule::find_best_split_value_small_q(const Data& data,
                                                               double& best_value,
                                                               size_t& best_var,
                                                               double& best_decrease,
+                                                              bool& best_send_missing_left,
                                                               const std::vector<double>& responses_by_sample,
                                                               const std::vector<std::vector<size_t>>& samples) {
   std::vector<double> possible_split_values;
-  data.get_all_values(possible_split_values, samples[node], var);
+  std::vector<size_t> sorted_samples;
+  data.get_all_values(possible_split_values, sorted_samples, samples[node], var);
 
   // Try next variable if all equal for this
   if (possible_split_values.size() < 2) {
     return;
   }
 
-  // Remove largest value because no split possible
-  possible_split_values.pop_back();
-
   // Initialize with 0m if not in memory efficient mode, use pre-allocated space
-  size_t num_splits = possible_split_values.size();
-  double* sums_right = sums;
-  size_t* n_right = counter;
+  size_t num_splits = possible_split_values.size() - 1;
 
-  std::fill(sums_right, sums_right + num_splits, 0);
-  std::fill(n_right, n_right + num_splits, 0);
+  std::fill(counter, counter + num_splits, 0);
+  std::fill(weight_sums, weight_sums + num_splits, 0);
+  std::fill(sums, sums + num_splits, 0);
   std::fill(num_small_z, num_small_z + num_splits, 0);
   std::fill(sums_z, sums_z + num_splits, 0);
   std::fill(sums_z_squared, sums_z_squared + num_splits, 0);
+  size_t n_missing = 0;
+  double weight_sum_missing = 0;
+  double sum_missing = 0;
+  double sum_z_missing = 0;
+  double sum_z_squared_missing = 0;
+  size_t num_small_z_missing = 0;
 
-  // Sum in right child and possible split
-  for (auto& sample : samples[node]) {
-    double value = data.get(sample, var);
-    double label = responses_by_sample[sample];
+  size_t split_index = 0;
+  for (size_t i = 0; i < num_samples - 1; i++) {
+    size_t sample = sorted_samples[i];
+    size_t next_sample = sorted_samples[i + 1];
+    double sample_value = data.get(sample, var);
     double z = data.get_instrument(sample);
+    double sample_weight = data.get_weight(sample);
 
-    // Count samples until split_value reached
-    for (size_t i = 0; i < num_splits; ++i) {
-      if (value > possible_split_values[i]) {
-        ++n_right[i];
-        sums_right[i] += label;
-        sums_z[i] += z;
-        sums_z_squared[i] += z * z;
-        if (z < mean_node_z) {
-          ++num_small_z[i];
-        }
-      } else {
-        break;
+    if (std::isnan(sample_value)) {
+      weight_sum_missing += sample_weight;
+      sum_missing += sample_weight * responses_by_sample[sample];
+      ++n_missing;
+
+      sum_z_missing += sample_weight * z;
+      sum_z_squared_missing += sample_weight * z * z;
+      if (z < mean_node_z) {
+        ++num_small_z_missing;
       }
+    } else {
+      weight_sums[split_index] += sample_weight;
+      sums[split_index] += sample_weight * responses_by_sample[sample];
+      ++counter[split_index];
+
+      sums_z[split_index] += sample_weight * z;
+      sums_z_squared[split_index] += sample_weight * z * z;
+      if (z < mean_node_z) {
+        ++num_small_z[split_index];
+      }
+    }
+
+    double next_sample_value = data.get(next_sample, var);
+    // if the next sample value is different, including the transition (..., NaN, Xij, ...)
+    // then move on to the next bucket (all logical operators with NaN evaluates to false by default)
+    if (sample_value != next_sample_value && !std::isnan(next_sample_value)) {
+      ++split_index;
     }
   }
 
+  size_t n_left = n_missing;
+  double weight_sum_left = weight_sum_missing;
+  double sum_left = sum_missing;
+  double sum_left_z = sum_z_missing;
+  double sum_left_z_squared = sum_z_squared_missing;
+  size_t num_left_small_z = num_small_z_missing;
+
   // Compute decrease of impurity for each possible split.
-  for (size_t i = 0; i < num_splits; ++i) {
-    size_t n_left = num_samples - n_right[i];
-
-    // Stop if the right child does not contain enough z
-    // values below and above the parent's mean.
-    size_t num_right_small_z = num_small_z[i];
-    size_t num_right_large_z = n_right[i] - num_right_small_z;
-    if (num_right_small_z < min_node_size || num_right_large_z < min_node_size) {
-      break;
+  for (bool send_left : {true, false}) {
+    if (!send_left) {
+      // A normal split with no NaNs, so we can stop early.
+      if (n_missing == 0) {
+        break;
+      }
+      // It is not necessary to adjust n_right or sum_right as the the missing
+      // part is included in the total sum.
+      n_left = 0;
+      weight_sum_left = 0;
+      sum_left = 0;
+      sum_left_z = 0;
+      sum_left_z_squared = 0;
+      num_left_small_z = 0;
     }
 
-    // Skip this split if the left child does not contain enough
-    // z values below and above the parent's mean.
-    size_t num_left_small_z = num_node_small_z - num_right_small_z;
-    size_t num_left_large_z = n_left - num_left_small_z;
-    if (num_left_small_z < min_node_size || num_left_large_z < min_node_size) {
-      continue;
-    }
+    for (size_t i = 0; i < num_splits; ++i) {
+      // not necessary to evaluate sending right when splitting on NaN.
+      if (i == 0 && !send_left) {
+        continue;
+      }
 
-    // Calculate relevant quantities for the right child.
-    double sum_right = sums_right[i];
-    double sum_right_z = sums_z[i];
-    double sum_right_z_squared = sums_z_squared[i];
-    double size_right = sum_right_z_squared - sum_right_z * sum_right_z / (double) n_right[i];
+      n_left += counter[i];
+      num_left_small_z += num_small_z[i];
+      weight_sum_left += weight_sums[i];
+      sum_left += sums[i];
+      sum_left_z += sums_z[i];
+      sum_left_z_squared += sums_z_squared[i];
 
-    // Skip this split if the right child's variance is too small.
-    if (size_right < min_child_size || (imbalance_penalty > 0.0 && size_right == 0)) {
-      continue;
-    }
+      // Skip this split if the left child does not contain enough
+      // z values below and above the parent's mean.
+      size_t num_left_large_z = n_left - num_left_small_z;
+      if (num_left_small_z < min_node_size || num_left_large_z < min_node_size) {
+        continue;
+      }
 
-    // Calculate relevant quantities for the left child.
-    double sum_left = sum_node - sum_right;
-    double sum_left_z = sum_node_z - sum_right_z;
-    double sum_left_z_squared = sum_node_z_squared - sum_right_z_squared;
-    double size_left = sum_left_z_squared - sum_left_z * sum_left_z / (double) n_left;
+      // Stop if the right child does not contain enough z values below
+      // and above the parent's mean.
+      size_t n_right = num_samples - n_left;
+      size_t num_right_small_z = num_node_small_z - num_left_small_z;
+      size_t num_right_large_z = n_right - num_right_small_z;
+      if (num_right_small_z < min_node_size || num_right_large_z < min_node_size) {
+        break;
+      }
 
-    // Skip this split if the left child's variance is too small.
-    if (size_left < min_child_size || (imbalance_penalty > 0.0 && size_left == 0)) {
-      continue;
-    }
+      // Calculate relevant quantities for the left child.
+      double size_left = sum_left_z_squared - sum_left_z * sum_left_z / weight_sum_left;
+      // Skip this split if the left child's variance is too small.
+      if (size_left < min_child_size || (imbalance_penalty > 0.0 && size_left == 0)) {
+        continue;
+      }
 
-    // Calculate the decrease in impurity.
-    double decrease = sum_left * sum_left / (double) n_left + sum_right * sum_right / (double) n_right[i];
-    decrease -= imbalance_penalty * (1.0 / size_left + 1.0 / size_right);
+      // Calculate relevant quantities for the left child.
+      double weight_sum_right = weight_sum_node - weight_sum_left;
+      double sum_right = sum_node - sum_left;
+      double sum_right_z_squared = sum_node_z_squared - sum_left_z_squared;
+      double sum_right_z = sum_node_z - sum_left_z;
+      double size_right = sum_right_z_squared - sum_right_z * sum_right_z / weight_sum_right;
 
-    // Save this split if it is the best seen so fa.
-    if (decrease > best_decrease) {
-      best_value = possible_split_values[i];
-      best_var = var;
-      best_decrease = decrease;
+      // Skip this split if the right child's variance is too small.
+      if (size_right < min_child_size || (imbalance_penalty > 0.0 && size_right == 0)) {
+        continue;
+      }
+
+      // Calculate the decrease in impurity.
+      double decrease = sum_left * sum_left / weight_sum_left + sum_right * sum_right / weight_sum_right;
+      // Penalize splits that are too close to the edges of the data.
+      decrease -= imbalance_penalty * (1.0 / size_left + 1.0 / size_right);
+
+      // Save this split if it is the best seen so fa.
+      if (decrease > best_decrease) {
+        best_value = possible_split_values[i];
+        best_var = var;
+        best_decrease = decrease;
+        best_send_missing_left = send_left;
+      }
     }
   }
 }
@@ -233,6 +298,7 @@ void InstrumentalSplittingRule::find_best_split_value_large_q(const Data& data,
                                                               size_t node,
                                                               size_t var,
                                                               size_t num_samples,
+                                                              double weight_sum_node,
                                                               double sum_node,
                                                               double mean_node_z,
                                                               size_t num_node_small_z,
@@ -247,6 +313,7 @@ void InstrumentalSplittingRule::find_best_split_value_large_q(const Data& data,
   // Set counters to 0
   size_t num_unique = data.get_num_unique_data_values(var);
   std::fill(counter, counter + num_unique, 0);
+  std::fill(weight_sums, weight_sums + num_unique, 0);
   std::fill(sums, sums + num_unique, 0);
   std::fill(num_small_z, num_small_z + num_unique, 0);
   std::fill(sums_z, sums_z + num_unique, 0);
@@ -255,18 +322,21 @@ void InstrumentalSplittingRule::find_best_split_value_large_q(const Data& data,
   for (auto& sample : samples[node]) {
     size_t i = data.get_index(sample, var);
     double z = data.get_instrument(sample);
+    double sample_weight = data.get_weight(sample);
 
-    sums[i] += responses_by_sample[sample];
+    weight_sums[i] += sample_weight;
+    sums[i] += sample_weight * responses_by_sample[sample];
     ++counter[i];
 
-    sums_z[i] += z;
-    sums_z_squared[i] += z * z;
+    sums_z[i] += sample_weight * z;
+    sums_z_squared[i] += sample_weight * z * z;
     if (z < mean_node_z) {
       ++num_small_z[i];
     }
   }
 
   size_t n_left = 0;
+  double weight_sum_left = 0;
   double sum_left = 0;
   double sum_left_z = 0.0;
   double sum_left_z_squared = 0.0;
@@ -281,6 +351,7 @@ void InstrumentalSplittingRule::find_best_split_value_large_q(const Data& data,
 
     n_left += counter[i];
     num_left_small_z += num_small_z[i];
+    weight_sum_left += weight_sums[i];
     sum_left += sums[i];
     sum_left_z += sums_z[i];
     sum_left_z_squared += sums_z_squared[i];
@@ -302,17 +373,18 @@ void InstrumentalSplittingRule::find_best_split_value_large_q(const Data& data,
     }
 
     // Calculate relevant quantities for the left child.
-    double size_left = sum_left_z_squared - sum_left_z * sum_left_z / (double) n_left;
-
+    double size_left = sum_left_z_squared - sum_left_z * sum_left_z / weight_sum_left;
     // Skip this split if the left child's variance is too small.
     if (size_left < min_child_size || (imbalance_penalty > 0.0 && size_left == 0)) {
       continue;
     }
 
     // Calculate relevant quantities for the left child.
+    double weight_sum_right = weight_sum_node - weight_sum_left;
+    double sum_right = sum_node - sum_left;
     double sum_right_z_squared = sum_node_z_squared - sum_left_z_squared;
     double sum_right_z = sum_node_z - sum_left_z;
-    double size_right = sum_right_z_squared - sum_right_z * sum_right_z / (double) n_right;
+    double size_right = sum_right_z_squared - sum_right_z * sum_right_z / weight_sum_right;
 
     // Skip this split if the right child's variance is too small.
     if (size_right < min_child_size || (imbalance_penalty > 0.0 && size_right == 0)) {
@@ -320,9 +392,7 @@ void InstrumentalSplittingRule::find_best_split_value_large_q(const Data& data,
     }
 
     // Calculate the decrease in impurity.
-    double sum_right = sum_node - sum_left;
-    double decrease = sum_left * sum_left / (double) n_left + sum_right * sum_right / (double) n_right;
-
+    double decrease = sum_left * sum_left / weight_sum_left + sum_right * sum_right / weight_sum_right;
     // Penalize splits that are too close to the edges of the data.
     decrease -= imbalance_penalty * (1.0 / size_left + 1.0 / size_right);
 
