@@ -39,9 +39,9 @@ public:
                        const std::vector<double>& responses_by_sample,
                        const std::vector<std::vector<size_t>>& samples,
                        std::vector<size_t>& split_vars,
-                       std::vector<double>& split_values);
+                       std::vector<double>& split_values,
+                       std::vector<bool>& send_missing_left);
 
-private:
   void find_best_split_value_small_q(const Data& data,
                                      size_t node, size_t var, size_t num_classes, size_t* class_counts,
                                      size_t size_node,
@@ -49,6 +49,7 @@ private:
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
+                                     bool& best_send_missing_left,
                                      const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
 
@@ -62,6 +63,7 @@ private:
                                      const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
 
+private:
   size_t num_classes;
 
   double alpha;
